@@ -27,6 +27,7 @@ import pygtk
 pygtk.require('2.0')
 import gobject
 import gtk
+import math
 import sys
 import traceback
 import os
@@ -43,6 +44,11 @@ def update2(mapWidget):
   mapWidget.checkForRedraw()
   return(True)
 
+def simplePythagoreanDistance(x1, y1, x2, y2):
+    dx = x2 - x1
+    dy = y2 - y1
+    return math.sqrt(dx**2 + dy**2)
+
 class MapWidget(gtk.Widget):
   __gsignals__ = { \
     'realize': 'override',
@@ -62,10 +68,14 @@ class MapWidget(gtk.Widget):
 
     """ setting this both to 100 and 1 in mapView and gpsd fixes the arow orientation bug """
     self.timer1 = gobject.timeout_add(100, update1, self) #default 100
-    self.timer2 = gobject.timeout_add(10, update2, self) #default 10
+#    self.timer2 = gobject.timeout_add(10, update2, self) #default 10
     self.timer3 = None # will be used for timing long press events
     self.d = {} # List of data
     self.m = {} # List of modules
+    self.watches = {} # List of data change watches
+    self.maxWatchId = 0
+    self.watch('needRedraw', self._checkForRedrawCB) # react on redraw requests
+    # TODO: add a function for directly requesting redraw
 
     self.mapRotationAngle = 0 # in radians
     self.notMovingSpeed = 1 # in m/s
@@ -75,6 +85,9 @@ class MapWidget(gtk.Widget):
     self.redraw = True
 
     self.showRedrawTime = False
+    
+    # map center shifting variables
+    self.centerShift = (0,0)
 
     # alternative map drag variables
     self.altMapDragEnabled = False
@@ -124,34 +137,122 @@ class MapWidget(gtk.Widget):
       m.modrana = self # make this class accessible from modules
       m.dmod = self.dmod
 
+    # as the options module should be already loaded, we can update the viewport
+    self._updateViewport()
+
+    # run what needs to be done before firstTime is called
+    self._modulesLoadedPreFirstTime()
+
     start = time.clock()
     for m in self.m.values():
       m.firstTime()
+
+    # run what needs to be done after firstTime is called
+    self._modulesLoadedPostFirstTime()
+
+    print "Initialization complete in %1.2f ms" % (1000 * (time.clock() - start))
+
+  def _modulesLoadedPreFirstTime(self):
+    """this is run after all the modules have been loaded,
+    but before their first time is called"""
+    self._updateCenteringShiftCB()
+
+    """to nly update values needed for map drawing when something changes
+       * window is resized
+       * user switches something rleated in options
+       * etc.
+       we use the key watching mechanism
+       once a related key is changed, we update all the values
+       """
+    # watch both centering shift related variables
+    self.watch('posShiftAmount', self._updateCenteringShiftCB)
+    self.watch('posShiftDirection', self._updateCenteringShiftCB)
+    # also watch the viewport
+    self.watch('viewport', self._updateCenteringShiftCB)
+    # and map scaling
+    self.watch('mapScale', self._updateCenteringShiftCB)
+
+  def _modulesLoadedPostFirstTime(self):
+    """this is run after all the modules have been loaded,
+    after before their first time is called"""
+    
     # check if redrawing time should be printed to terminal
     if 'showRedrawTime' in self.d and self.d['showRedrawTime'] == True:
       self.showRedrawTime = True
-    print "Initialization complete in %1.2f ms" % (1000 * (time.clock() - start))
-      
+
   def beforeDie(self):
     print "Shutting-down modules"
     for m in self.m.values():
       m.shutdown()
     time.sleep(2) # leave some times for threads to shut down
     print "Shuttdown complete"
-  
+    
+  ## OPTIONS SETTING AND WATCHING ##
+
+  def get(self, name, default=None):
+    """Get an item of data"""
+    return(self.d.get(name, default))
+
+  def set(self, name, value, save=False):
+    """Set an item of data"""
+    self.d[name] = value
+    """options are normally saved on shutdown,
+    but for some data we want to make sure they are stored and not
+    los for example becuase of power outage/empty battery, etc."""
+    if save:
+      options = self.m.get('options')
+      if options:
+        options.save()
+    if name in self.watches.keys():
+      self._notifyWatcher(name, value)
+
+  def watch(self, key, callback, *args):
+    """add a callback on an options key"""
+    nrId = self.maxWatchId + 1
+    id = "%d_%s" % (nrId,key)
+    self.maxWatchId = nrId # TODO: recycle ids ? (alla PID)
+    if key not in self.watches:
+      self.watches[key] = [] # create the initial list
+    self.watches[key].append((id,callback,args))
+    return id
+
+  def removeWatch(self, id):
+    """remove watch specified by the given watch id"""
+    (nrId, key) = id.split('_')
+
+    if key in self.watches:
+      remove = lambda x:x==id
+      self.watches[key][:] = [x for x in self.watches[key] if not remove(x)]
+    else:
+      print "modRana: cant remove watch - key does not exist, watchId:", id
+
+  def _notifyWatcher(self, key, value):
+    """run callbacks registered on an options key"""
+    callbacks = self.watches.get(key, None)
+    if callbacks:
+      for item in callbacks:
+        (id,callback,args) = item
+        oldValue = self.get(key, None)
+        if callback:
+          callback(key,value,oldValue, *args)
+        else:
+          print "invalid watcher callback :", callback
+
   def update(self):
     for m in self.m.values():
       m.update()
 
-  def checkForRedraw(self):
-    # check if redrawing is enabled
-    if(self.d.get("needRedraw", False)):
+  def _checkForRedrawCB(self, key, oldValue, newValue):
+    """react to redraw requests"""
+    if newValue == True:
       self.forceRedraw()
       
   def forceRedraw(self):
     """Make the window trigger a draw event.
     TODO: consider replacing this if porting pyroute to another platform"""
     self.d['needRedraw'] = False
+    """ alter directly, no need to notificate
+    about returning the key to default state"""
     if self.redraw:
       try:
         self.window.invalidate_rect((0,0,self.rect.width,self.rect.height),False)
@@ -182,7 +283,7 @@ class MapWidget(gtk.Widget):
       
   def handleDrag(self,x,y,dx,dy,startX,startY,msDuration):
     # check if centering is on
-    if self.d.get("centred",True):
+    if self.get("centred",True):
       fullDx = x - startX
       fullDy = y - startY
       distSq = fullDx * fullDx + fullDy * fullDy
@@ -190,14 +291,7 @@ class MapWidget(gtk.Widget):
       -> like this, centering is not dsabled by pressing buttons"""
       if self.centeringDisableTreshold:
         if distSq > self.centeringDisableTreshold:
-          self.d["centred"] = False # turn off centering after dragging the map (like in TangoGPS)
-          m = self.m.get('messages', None)
-          if m:
-            """the device module might have to update button state if it has an
-            external application menu with toggle buttons
-            eq. if centering state changes by dragging the map, it needs to update its
-            toggle button state"""
-            m.sendMessage('device:updateAppMenu')
+          self.set("centred", False) # turn off centering after dragging the map (like in TangoGPS)
           self.d["needRedraw"] = True
     else:
       if self.altMapDragEnabled:
@@ -232,6 +326,46 @@ class MapWidget(gtk.Widget):
     default value = 2048
     """
     self.centeringDisableTreshold = treshold
+    
+  def _updateCenteringShiftCB(self, key=None, oldValue=None, newValue=None):
+    """update shifted centering amount
+
+    this method is called if posShiftAmount or posShiftDirection
+    are set and also once at startup"""
+    (sx,sy,sw,sh) = self.get('viewport')
+    x=0
+    y=0
+    shiftAmount = self.d.get('posShiftAmount', 0.75)
+    """this value might show up as string, so we convert it to float, just to be sure"""
+    floatShiftAmount = float(shiftAmount)
+    shiftDirection = self.d.get('posShiftDirection', "down")
+    if shiftDirection:
+      if shiftDirection == "down":
+        y =  sh * 0.5 * floatShiftAmount
+      elif shiftDirection == "up":
+        y =  - sh * 0.5 * floatShiftAmount
+      elif shiftDirection == "left":
+        x =  - sw * 0.5 * floatShiftAmount
+      elif shiftDirection == "right":
+        x =  + sw * 0.5 * floatShiftAmount
+      """ we dont need to do anything if direction is set to don't shift (False)
+      - 0,0 will be used """
+    self.centerShift = (x,y)
+    
+    # update the viewport expansion variable
+    scale = int(self.get('mapScale', 1))
+    tileSide = 256
+    mapTiles = self.m.get('mapTiles')
+    if mapTiles: # check the mapTiles for tile side length in pixels, if available
+      tileSide = mapTiles.tileSide
+    tileSide = tileSide * scale # apply any possible scaling
+    (centerX,centerY) = ((sw/2.0),(sh/2.0))
+    ulCenterDistance = simplePythagoreanDistance(0, 0, centerX, centerY)
+    centerLLdistance = simplePythagoreanDistance(centerX, centerY, sw, sh)
+    diagonal = max(ulCenterDistance, centerLLdistance)
+    add = int(math.ceil(float(diagonal)/tileSide))
+    self.expandViewportTiles = add
+
 
   def draw(self, cr, event):
     """ re/Draw the modrana GUI """
@@ -261,29 +395,14 @@ class MapWidget(gtk.Widget):
         proj = self.m['projection']
         (lat, lon) = (proj.lat,proj.lon)
         (x1,y1) = proj.ll2xy(lat, lon)
-        (sx,sy,sw,sh) = self.d.get('viewport')
-        x=0
-        y=0
-        shiftAmount = self.d.get('posShiftAmount', 0.75)
-        """this value might show up as string, so we convert it to float, just to be sure"""
-        floatShiftAmount = float(shiftAmount)
-        shiftDirection = self.d.get('posShiftDirection', "down")
-        if shiftDirection:
-          if shiftDirection == "down":
-            y =  sh * 0.5 * floatShiftAmount
-          elif shiftDirection == "up":
-            y =  - sh * 0.5 * floatShiftAmount
-          elif shiftDirection == "left":
-            x =  - sw * 0.5 * floatShiftAmount
-          elif shiftDirection == "right":
-            x =  + sw * 0.5 * floatShiftAmount
-          # we dont need to do anything if direction is set to don't shift (False)
+
+        (x,y) = self.centerShift
         cr.translate(x,y)
         cr.save()
         # get the speed and angle
         speed = self.d.get('speed', 0)
         angle = self.d.get('bearing', 0)
-
+        
         """
         only if current direction angle and speed are known,
         submit a new angle
@@ -292,10 +411,9 @@ class MapWidget(gtk.Widget):
         """
         if angle and speed:
           if speed > self.notMovingSpeed: # do we look like we are moving ?
-            angle = 360 - angle
-            self.mapRotationAngle = radians(angle)
+            self.mapRotationAngle = angle
         cr.translate(x1,y1) # translate to the rotation center
-        cr.rotate(self.mapRotationAngle) # do the rotation
+        cr.rotate(radians(360 - self.mapRotationAngle)) # do the rotation
         cr.translate(-x1,-y1) # translate back
 
         # Draw the base map, the map overlays, and the screen overlays
@@ -486,6 +604,7 @@ class MapWidget(gtk.Widget):
     self.backingPixmap.draw_drawable(self.gc, self.window, 0, 0, 0,0,-1,-1)
     # initiate first drag
     self.staticMapDrag(shiftX, shiftY,dx,dy)
+    self.set('needRedraw',True)
 
   def staticMapDragEnd(self, event):
     """revert the changes neede for the drag"""
@@ -501,13 +620,13 @@ class MapWidget(gtk.Widget):
     self.setCurrentRedrawMethod()
 
     # redraw the whole screen
-    self.d['needRedraw'] = True
+    self.set('needRedraw',True)
 
   def staticMapDrag(self,shiftX,shiftY,dx,dy):
     """drag the map"""
     self.shift = (shiftX,shiftY,dx,dy)
     (x,y,w,h) = (self.rect.x, self.rect.y, self.rect.width, self.rect.height)
-    self.d['needRedraw'] = True
+    self.set('needRedraw',True)
 
   def staticMapPixmapDrag(self, cr, event):
     (x,y,w,h) = (self.rect.x, self.rect.y, self.rect.width, self.rect.height)
@@ -517,7 +636,7 @@ class MapWidget(gtk.Widget):
   def staticMapRevert(self):
     self.defaulMethodBindings()
     self.altMapDragEnabled = False
-    
+
   def do_realize(self):
     self.set_flags(self.flags() | gtk.REALIZED)
     self.window = gdk.Window( \
@@ -544,6 +663,9 @@ class MapWidget(gtk.Widget):
     pass
   def do_size_allocate(self, allocation):
     self.allocation = allocation
+    self.rect = self.allocation
+    self._updateViewport()
+    
     if self.flags() & gtk.REALIZED:
       self.window.move_resize(*allocation)
     newW = allocation[2]
@@ -556,10 +678,13 @@ class MapWidget(gtk.Widget):
     # notify all modules
     for m in self.m.values(): # enable resize handling by modules
         m.handleResize(newW, newH)
-#    self.forceRedraw() # redraw window contents after resize
+    self.forceRedraw() # redraw window contents after resize
+
+  def _updateViewport(self):
+    """update the current viewport in the global perzistent dictionary"""
+    self.set('viewport', (self.rect.x, self.rect.y, self.rect.width, self.rect.height))
+
   def _expose_cairo(self, event, cr):
-    self.rect = self.allocation
-    self.d['viewport'] = (self.rect.x, self.rect.y, self.rect.width, self.rect.height)
     #self.modules['projection'].setView( \
     #  self.rect.x, 
     #  self.rect.y, 
